from pkg_resources import get_distribution, DistributionNotFound


__version__ = "0.1.0dev0"


__title__ = "xicor"
__description__ = "xi correlation for tied data"
__uri__ = "https://github.com/czbiohub/xicor"
__doc__ = __description__ + " <" + __uri__ + ">"

__author__ = "Phoenix Logan"
__email__ = "phoenix.logan@czbiohub.org"

__license__ = "MIT License"
__copyright__ = "Copyright (c) 2020 Chan Zuckerberg Biohub"


try:
    VERSION = get_distribution(__name__).version
except DistributionNotFound:  # pragma: no cover
    try:
        from .version import version as VERSION  # noqa
    except ImportError:  # pragma: no cover
        raise ImportError(
            "Failed to find (autogenerated) version.py. "
            "This might be because you are installing from GitHub's tarballs, "
            "use the PyPI ones."
        )
